import phonenumbers
from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

from ocyan.core.fender import config


def get_from_email(request, form):  # pylint: disable=unused-argument
    return config.get("contact_form", "email_from", settings.DEFAULT_FROM_EMAIL)


def get_to_email(request, form):  # pylint: disable=unused-argument
    to_email = config.get("contact_form", "email_to", settings.DEFAULT_TO_EMAIL)
    return to_email if isinstance(to_email, (list, tuple)) else [to_email]


def validate_phone_number(phone_number):
    """
    Checks phone number with all the country codes and raises error if its invalid
    across all the country codes or supported regions.
    """
    cleaned_number = "".join(c for c in phone_number if c.isdigit() or c == "+")
    if not cleaned_number.startswith("+"):
        all_regions = phonenumbers.SUPPORTED_REGIONS
        for region in all_regions:
            try:
                parsed_number = phonenumbers.parse(cleaned_number, region)
                if phonenumbers.is_valid_number(parsed_number):
                    return
            except phonenumbers.NumberParseException:
                continue
    else:
        try:
            parsed_number = phonenumbers.parse(cleaned_number)
            if phonenumbers.is_valid_number(parsed_number):
                return
        except phonenumbers.NumberParseException:
            pass

    raise ValidationError(_("The phone number entered is not valid."), code="invalid")



from django.views.generic import TemplateView
from django.contrib import messages
from django.contrib.sites.shortcuts import get_current_site
from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from django.utils.translation import gettext as _
from django.views.decorators.http import require_http_methods
from django.urls import reverse
from django.shortcuts import redirect

from oscar.core.utils import redirect_to_referrer
from ocyan.core.fender import config
from ocyan.main.loading import get_ocyan_class, get_ocyan_classes

ContactForm = get_ocyan_class("contact_form.forms", "ContactForm")
# pylint: disable=unbalanced-tuple-unpacking
get_from_email, get_to_email = get_ocyan_classes(
    "contact_form.utils", ["get_from_email", "get_to_email"]
)


@require_http_methods(["POST"])
def post_contact_form(request):
    form = ContactForm(request.POST, request=request)

    if form.is_valid():
        context = {
            "website_url": request.build_absolute_uri(),
            "form_data": form.cleaned_data,
        }

        html_message = render_to_string("contact_form/contact_email.html", context)
        text_message = render_to_string("contact_form/contact_email.txt", context)

        site = get_current_site(request)
        site_name = site.name or config.get("django", "name")

        subject = _("Contact form email from %s") % site_name
        msg = EmailMultiAlternatives(
            subject,
            text_message,
            from_email=get_from_email(request, form),
            to=get_to_email(request, form),
            reply_to=[form.cleaned_data["email_from"]],
        )
        msg.attach_alternative(html_message, "text/html")
        msg.send()

        request.session["contact_form_submitted"] = True
        messages.add_message(request, messages.SUCCESS, _("Message sent"))
        return redirect(reverse("contact_form:contact-form-thank-you"))
    else:
        request.session["contact_form_post_data"] = request.POST
        messages.add_message(
            request,
            messages.ERROR,
            _("An error occured in the contact form: %s") % form.errors.as_text(),
        )
        return redirect_to_referrer(request, "contact_form:contact-form-handler")


class ContactFormThankYou(TemplateView):
    template_name = "contact_form/thank_you.html"

    def get(self, request, *args, **kwargs):
        contact_form_submitted = request.session.pop("contact_form_submitted", False)
        if contact_form_submitted:
            return super().get(request, *args, **kwargs)
        else:
            return redirect(settings.CONTACT_REDIRECT_URL)



# -*- coding: utf-8 -*-
from django.conf.urls.i18n import i18n_patterns
from django.urls import include, path

from ocyan.core.fender import config
from ocyan.core.utils import merge_dicts
from ocyan.plugin.google.constants import CAPTCHA_ENABLED
from ocyan.plugin.wagtail.block_plugin import BlockPluginConfig

from .constants import APP_CODE

SHOP_BASE_URL = config.get("oscar", "shop_base_url", "shop")


class ContactFormConfig(BlockPluginConfig):
    def contribute_to_installed_apps(self, installed_apps):
        installed_apps.apps.append("phonenumber_field")
        installed_apps.ocyan.append("ocyan.plugin.contact_form")
        installed_apps.wagtail.register_extension(
            "ocyan.plugin.contact_form.contact_form_coyote_extension",
            "ocyan.plugin.coyote",
        )

        if CAPTCHA_ENABLED:
            installed_apps.apps.append("django_recaptcha")

        return installed_apps

    def contribute_to_urlpatterns(self, urlpatterns):
        if config.i18n_enabled:
            return (
                i18n_patterns(
                    path(
                        r"%s/" % SHOP_BASE_URL,
                        include("ocyan.plugin.contact_form.urls", namespace=APP_CODE),
                    ),
                    prefix_default_language=False,
                )
                + urlpatterns
            )

        return [
            path(
                r"%s/" % SHOP_BASE_URL,
                include("ocyan.plugin.contact_form.urls", namespace=APP_CODE),
            )
        ] + urlpatterns

    def contribute_to_blocks(self, blocks):
        from .blocks import ContactFormBlock

        return blocks + [("Contact", ContactFormBlock())]

    def contribute_to_settings(self, settings):
        CONTACT_FORM_SETTINGS = {
            "CONTACT_REDIRECT_URL": "/",
        }

        return merge_dicts(settings, CONTACT_FORM_SETTINGS)

    def get_smoketest_modules(self):
        return ["ocyan.plugin.contact_form.tests"]



# pylint: disable= E0632
from django.urls import path

from ocyan.main.loading import get_ocyan_class

from .constants import APP_CODE

post_contact_form = get_ocyan_class("contact_form.views", "post_contact_form")
ContactFormThankYou = get_ocyan_class("contact_form.views", "ContactFormThankYou")


app_name = APP_CODE
urlpatterns = [
    path("contact-form/", post_contact_form, name="contact-form-handler"),
    path(
        "contact-form/thank-you/",
        ContactFormThankYou.as_view(),
        name="contact-form-thank-you",
    ),
]



from django.utils.translation import gettext as _
from wagtail import blocks

from ocyan.main.loading import get_ocyan_class, get_baseclass
from ocyan.plugin.coyote.blocks.partials import CoyoteStructBlock


BaseContactPageContentStreamBlock = get_baseclass(
    "coyote.stream_blocks",
    "ContactPageContentStreamBlock",
    "ocyan.plugin.contact_form.contact_form_coyote_extension",
)
ContactForm = get_ocyan_class("contact_form.forms", "ContactForm")


class ContactFormBlock(CoyoteStructBlock):
    title = blocks.CharBlock(blank=True, required=False, default="")
    body = blocks.RichTextBlock(blank=True, required=False)

    class Meta:
        template = "coyote/blocks/content/contact.html"
        icon = "form"
        help_text = _("A contact form has been added to this page!")
        label = _("Contact form")

    def get_context(self, value, parent_context=None):
        context = super().get_context(value, parent_context=parent_context)
        request = context.get("request")
        post_data = request.session.pop("contact_form_post_data", None)
        form = ContactForm(post_data, request=request)
        if post_data is not None:
            form.is_valid()
        context["form"] = form
        return context


class ContactPageContentStreamBlock(BaseContactPageContentStreamBlock):
    blogs = ContactFormBlock(icon="pick")



from django.core.exceptions import ValidationError
from django.test import TestCase
from django.test.client import RequestFactory

from ocyan.main.loading import get_ocyan_class

from .utils import validate_phone_number

ContactForm = get_ocyan_class("contact_form.forms", "ContactForm")


class ContactFormSmokeTestCase(TestCase):
    def test_backwards_compatibility(self):
        """
        We added the request as form argument, but if projects have the form overidden without extending
        the form class from this plugin, it will break. This test makes sure that projects that have this
        break in Jenkins and that we don't figure this out once released.
        """
        request = RequestFactory().get("/")
        try:
            ContactForm(request=request)
        except TypeError as e:
            if "unexpected keyword argument 'request'" in str(e):
                self.fail(
                    "ContactForm now receives a request object as argument, adjust the form by adding self.request = kwargs.pop('request'', None) to the init method (before calling super)"
                )


class PhoneNumberValidationTestCase(TestCase):
    def test_phone_number_validation(self):
        validate_phone_number("123-456-7890")
        validate_phone_number("(555) 123-4567")
        validate_phone_number("9876543210")
        validate_phone_number("12345")
        validate_phone_number("+442071234567")
        validate_phone_number("01234567890")

        with self.assertRaises(ValidationError):
            validate_phone_number("123")

        with self.assertRaises(ValidationError):
            validate_phone_number("SDFFdd")

        with self.assertRaises(ValidationError):
            validate_phone_number("SDFF++dd")

        with self.assertRaises(ValidationError):
            validate_phone_number("+0")



# -*- coding: utf-8 -*-
from pathlib import Path

from setuptools import find_packages, setup

install_requires: list = [
    "ocyan.core",
    "ocyan.plugin.wagtail",
    "ocyan.plugin.wagtail_content_page",
    "ocyan.plugin.google",
    "django-recaptcha",
    "django-phonenumber-field",
]

extras_require: dict = {
    "test": [
        "ocyan.plugin.testing",
        "wheel",
        "empty_testproject",
        "pylint-django",
        "black",
        "vdt.versionplugin.wheel",
        "coverage",
        "ocyan.plugin.coyote",
    ]
}

PACKAGE_CLASSIFIERS = [
    "Framework :: Ocyan",
    "Environment :: Plugins",
]

README = Path(__file__).with_name("README.md").read_text(encoding="utf-8")

setup(
    name="ocyan.plugin.contact_form",
    version="0.2.1",
    url="https://git.mandelblog.com/mandel-plugins/ocyan.plugin.contact_form",
    author="Motolani Olaiya",
    author_email="motolaniolaiya@gmail.com",
    description=(
        "Create contact forms on any page with reCAPTCHA from the google plugin,"
        " for spam protection."
    ),
    long_description=README,
    long_description_content_type="text/markdown",
    license="Proprietary",
    packages=find_packages(),
    include_package_data=True,
    python_requires=">=3.10",
    install_requires=install_requires,
    classifiers=PACKAGE_CLASSIFIERS,
    extras_require=extras_require,
)



from django import forms
from django.utils.translation import gettext_lazy as _
from django_recaptcha.fields import ReCaptchaField

from ocyan.plugin.google.constants import CAPTCHA_ENABLED
from ocyan.plugin.google.recaptcha import RECAPTCHA_WIDGET

from .utils import validate_phone_number


class ContactForm(forms.Form):
    name = forms.CharField(label=_("Name"))
    phonenumber = forms.CharField(
        label=_("Phone number"), required=False, validators=[validate_phone_number]
    )
    email_from = forms.EmailField(label=_("Email address"))
    message = forms.CharField(widget=forms.Textarea, label=_("Message"))

    name.widget.attrs.update({"autocomplete": "name", "placeholder": _("Name")})
    phonenumber.widget.attrs.update(
        {"autocomplete": "tel", "type": "tel", "placeholder": _("Phone number")}
    )
    email_from.widget.attrs.update(
        {"autocomplete": "email", "placeholder": _("Email address")}
    )
    message.widget.attrs.update({"placeholder": _("Message")})

    if CAPTCHA_ENABLED:
        captcha = ReCaptchaField(widget=RECAPTCHA_WIDGET, label="")

    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop("request", None)
        super().__init__(*args, **kwargs)



from wagtail import blocks
from ocyan.main.loading import get_ocyan_class
from django.utils.translation import gettext_lazy as _


ContactForm = get_ocyan_class("contact_form.forms", "ContactForm")


class ContactFormBlock(blocks.StructBlock):
    title = blocks.CharBlock(blank=True, required=False, default="")

    class Meta:
        template = "contact_form/blocks/contact-form.html"
        icon = "form"
        help_text = _("A contact form has been added to this page!")
        label = _("Contact form")
        group = "Extra"

    def get_context(self, value, parent_context=None):
        context = super().get_context(value, parent_context=parent_context)
        request = context.get("request")
        post_data = request.session.pop("contact_form_post_data", None)
        form = ContactForm(post_data, request=request)
        if post_data is not None:
            form.is_valid()
        context["form"] = form
        return context
